@@ -74,7 +74,7 @@ SOFTWARE.
  *
  */
 
-int
+int _X_COLD
 SProcXListInputDevices(ClientPtr client)
 {
     REQUEST(xListInputDevicesReq);
@@ -119,9 +119,9 @@ SizeDeviceInfo(DeviceIntPtr d, int *namesize, int *size)
  */
 
 static void
-CopyDeviceName(char **namebuf, char *name)
+CopyDeviceName(char **namebuf, const char *name)
 {
-    char *nameptr = (char *) *namebuf;
+    char *nameptr = *namebuf;
 
     if (name) {
         *nameptr++ = strlen(name);
@@ -413,7 +413,7 @@ ProcXListInputDevices(ClientPtr client)
  *
  */
 
-void
+void _X_COLD
 SRepXListInputDevices(ClientPtr client, int size, xListInputDevicesReply * rep)
 {
     swaps(&rep->sequenceNumber);
