@@ -33,14 +33,14 @@
 #define ACPI_VIDEO_HEAD_END		(~0u)
 
 static void lnxCloseACPI(void);
-static pointer ACPIihPtr = NULL;
+static void *ACPIihPtr = NULL;
 PMClose lnxACPIOpen(void);
 
 /* in milliseconds */
 #define ACPI_REOPEN_DELAY 1000
 
 static CARD32
-lnxACPIReopen(OsTimerPtr timer, CARD32 time, pointer arg)
+lnxACPIReopen(OsTimerPtr timer, CARD32 time, void *arg)
 {
     if (lnxACPIOpen()) {
         TimerFree(timer);
@@ -74,28 +74,30 @@ lnxACPIGetEventFromOs(int fd, pmEvent * events, int num)
 
     /* Check that we have a video event */
     if (!strncmp(ev, "video", 5)) {
-        char *video = NULL;
         char *GFX = NULL;
         char *notify = NULL;
         char *data = NULL;      /* doesn't appear to be used in the kernel */
-        unsigned long int notify_l, data_l;
+        unsigned long int notify_l;
 
-        video = strtok(ev, " ");
+        strtok(ev, " ");
 
-        GFX = strtok(NULL, " ");
+        if (!(GFX = strtok(NULL, " ")))
+            return 0;
 #if 0
         ErrorF("GFX: %s\n", GFX);
 #endif
 
-        notify = strtok(NULL, " ");
+        if (!(notify = strtok(NULL, " ")))
+            return 0;
         notify_l = strtoul(notify, NULL, 16);
 #if 0
         ErrorF("notify: 0x%lx\n", notify_l);
 #endif
 
-        data = strtok(NULL, " ");
-        data_l = strtoul(data, NULL, 16);
+        if (!(data = strtok(NULL, " ")))
+            return 0;
 #if 0
+        data_l = strtoul(data, NULL, 16);
         ErrorF("data: 0x%lx\n", data_l);
 #endif
 
